import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _initializerDefineProperty from "@babel/runtime/helpers/esm/initializerDefineProperty";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _applyDecoratedDescriptor from "@babel/runtime/helpers/esm/applyDecoratedDescriptor";
import _initializerWarningHelper from "@babel/runtime/helpers/esm/initializerWarningHelper";
var _dec, _dec2, _dec3, _dec4, _dec5, _dec6, _class, _class2, _descriptor, _descriptor2, _descriptor3, _descriptor4, _descriptor5;
import { CameraUniform, CoordinateUniform, TYPES } from '@antv/l7-core';
import { inject, injectable } from 'inversify';
import 'reflect-metadata';

/**
 * 在渲染之前需要获取当前 Shader 所需 Uniform，例如：
 * 1. 从相机服务中获取 View & ProjectionMatrix，当前缩放等级等等
 * 2. 从坐标系服务中获取当前坐标系，例如是否是偏移坐标系
 *    @see https://yuque.antfin-inc.com/yuqi.pyq/fgetpa/doml91
 * 3. 当前 Layer 本身的样式属性
 */
var ShaderUniformPlugin = (_dec = injectable(), _dec2 = inject(TYPES.ICameraService), _dec3 = inject(TYPES.ICoordinateSystemService), _dec4 = inject(TYPES.IRendererService), _dec5 = inject(TYPES.IMapService), _dec6 = inject(TYPES.ILayerService), _dec(_class = (_class2 = /*#__PURE__*/function () {
  function ShaderUniformPlugin() {
    _classCallCheck(this, ShaderUniformPlugin);
    _initializerDefineProperty(this, "cameraService", _descriptor, this);
    _initializerDefineProperty(this, "coordinateSystemService", _descriptor2, this);
    _initializerDefineProperty(this, "rendererService", _descriptor3, this);
    _initializerDefineProperty(this, "mapService", _descriptor4, this);
    _initializerDefineProperty(this, "layerService", _descriptor5, this);
  }
  _createClass(ShaderUniformPlugin, [{
    key: "apply",
    value: function apply(layer) {
      var _this = this;
      var version = this.mapService.version;
      var mvp = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]; // default matrix (for gaode2.x)
      var sceneCenterMercator = [0, 0];
      var uniformBuffer;
      if (!this.rendererService.uniformBuffers[0]) {
        // Create a Uniform Buffer Object(UBO).
        uniformBuffer = this.rendererService.createBuffer({
          data: new Float32Array(16 * 4 + 4 * 7),
          isUBO: true
        });
        this.rendererService.uniformBuffers[0] = uniformBuffer;
      }
      layer.hooks.beforeRender.tap('ShaderUniformPlugin', function () {
        // @ts-ignore
        var offset = layer.getLayerConfig().tileOrigin;
        // 重新计算坐标系参数
        _this.coordinateSystemService.refresh(offset);
        if (version === 'GAODE2.x') {
          _this.setLayerCenter(layer);
          // @ts-ignore
          mvp = _this.mapService.map.customCoords.getMVPMatrix();
          // mvp = amapCustomCoords.getMVPMatrix()
          // @ts-ignore
          sceneCenterMercator = _this.mapService.getCustomCoordCenter();
          var _uniformBuffer = layer.getLayerUniformBuffer();
          _uniformBuffer.subData({
            offset: 0,
            data: new Uint8Array(new Float32Array([].concat(_toConsumableArray(mvp), _toConsumableArray(sceneCenterMercator))).buffer)
          });
        }
        var _this$rendererService = _this.rendererService.getViewportSize(),
          width = _this$rendererService.width,
          height = _this$rendererService.height;
        var _this$generateUBO = _this.generateUBO(mvp, sceneCenterMercator, width, height),
          data = _this$generateUBO.data,
          uniforms = _this$generateUBO.uniforms;
        if (_this.layerService.alreadyInRendering && _this.rendererService.uniformBuffers[0]) {
          var renderUniformBuffer = _this.rendererService.uniformBuffers[0];
          // Update only once since all models can share one UBO.
          renderUniformBuffer.subData({
            offset: 0,
            data: data
          });
        }

        // For WebGL1. regl 
        var platformString = _this.rendererService.queryVerdorInfo();
        if (platformString === 'WebGL1') {
          layer.models.forEach(function (model) {
            model.addUniforms(_objectSpread(_objectSpread({}, uniforms), {}, {
              // TODO: move these 2 uniforms to PixelPickingPlugin
              u_PickingBuffer: layer.getLayerConfig().pickingBuffer || 0,
              // Tip: 当前地图是否在拖动
              u_shaderPick: Number(layer.getShaderPickStat())
            }));
          });
        }
      });
    }

    /**
     * 对于每个 layer 都有不同的几何中心点，因此在绘制每个 layer 的时候都需要重新设置
     * @param layer
     */
  }, {
    key: "setLayerCenter",
    value: function setLayerCenter(layer) {
      if (layer.coordCenter === undefined) {
        layer.coordCenter = layer.getSource().center;
      }
      if (this.mapService.setCoordCenter) {
        this.mapService.setCoordCenter(layer.coordCenter);
      }
    }
  }, {
    key: "generateUBO",
    value: function generateUBO(u_Mvp, sceneCenterMercator, width, height) {
      var _uniforms;
      var u_ProjectionMatrix = this.cameraService.getProjectionMatrix();
      var u_ViewMatrix = this.cameraService.getViewMatrix();
      var u_ViewProjectionMatrix = this.cameraService.getViewProjectionMatrix();
      var u_ModelMatrix = this.cameraService.getModelMatrix();
      var u_ViewportCenterProjection = this.coordinateSystemService.getViewportCenterProjection();
      var u_PixelsPerDegree = this.coordinateSystemService.getPixelsPerDegree();
      var u_Zoom = this.cameraService.getZoom();
      var u_PixelsPerDegree2 = this.coordinateSystemService.getPixelsPerDegree2();
      var u_ZoomScale = this.cameraService.getZoomScale();
      var u_PixelsPerMeter = this.coordinateSystemService.getPixelsPerMeter();
      var u_CoordinateSystem = this.coordinateSystemService.getCoordinateSystem();
      var u_CameraPosition = this.cameraService.getCameraPosition();
      var u_DevicePixelRatio = window.devicePixelRatio;
      var u_ViewportCenter = this.coordinateSystemService.getViewportCenter();
      var u_ViewportSize = [width, height];
      var u_FocalDistance = this.cameraService.getFocalDistance();
      var data = [].concat(_toConsumableArray(u_ViewMatrix), _toConsumableArray(u_ProjectionMatrix), _toConsumableArray(u_ViewProjectionMatrix), _toConsumableArray(u_ModelMatrix), _toConsumableArray(u_ViewportCenterProjection), _toConsumableArray(u_PixelsPerDegree), [
      // 4
      u_Zoom], _toConsumableArray(u_PixelsPerDegree2), [
      // 4
      u_ZoomScale], _toConsumableArray(u_PixelsPerMeter), [
      // 4
      u_CoordinateSystem], _toConsumableArray(u_CameraPosition), [
      // 4
      u_DevicePixelRatio], _toConsumableArray(u_ViewportCenter), u_ViewportSize, [
      // 4
      u_FocalDistance,
      // 1
      0, 0, 0]);
      return {
        data: data,
        uniforms: (_uniforms = {}, _defineProperty(_uniforms, CameraUniform.ProjectionMatrix, u_ProjectionMatrix), _defineProperty(_uniforms, CameraUniform.ViewMatrix, u_ViewMatrix), _defineProperty(_uniforms, CameraUniform.ViewProjectionMatrix, u_ViewProjectionMatrix), _defineProperty(_uniforms, CameraUniform.Zoom, u_Zoom), _defineProperty(_uniforms, CameraUniform.ZoomScale, u_ZoomScale), _defineProperty(_uniforms, CameraUniform.FocalDistance, u_FocalDistance), _defineProperty(_uniforms, CameraUniform.CameraPosition, u_CameraPosition), _defineProperty(_uniforms, CoordinateUniform.CoordinateSystem, u_CoordinateSystem), _defineProperty(_uniforms, CoordinateUniform.ViewportCenter, u_ViewportCenter), _defineProperty(_uniforms, CoordinateUniform.ViewportCenterProjection, u_ViewportCenterProjection), _defineProperty(_uniforms, CoordinateUniform.PixelsPerDegree, u_PixelsPerDegree), _defineProperty(_uniforms, CoordinateUniform.PixelsPerDegree2, u_PixelsPerDegree2), _defineProperty(_uniforms, CoordinateUniform.PixelsPerMeter, u_PixelsPerMeter), _defineProperty(_uniforms, CoordinateUniform.Mvp, u_Mvp), _defineProperty(_uniforms, "u_sceneCenterMercator", sceneCenterMercator), _defineProperty(_uniforms, "u_ViewportSize", u_ViewportSize), _defineProperty(_uniforms, "u_ModelMatrix", u_ModelMatrix), _defineProperty(_uniforms, "u_DevicePixelRatio", u_DevicePixelRatio), _uniforms)
      };
    }
  }]);
  return ShaderUniformPlugin;
}(), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "cameraService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class2.prototype, "coordinateSystemService", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class2.prototype, "rendererService", [_dec4], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor4 = _applyDecoratedDescriptor(_class2.prototype, "mapService", [_dec5], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor5 = _applyDecoratedDescriptor(_class2.prototype, "layerService", [_dec6], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
export { ShaderUniformPlugin as default };